@@ -27,7 +27,6 @@
 #include <cstring>
 #include <memory>
 #include "surface.h"
-#include "palette.h"
 #include "error.h"
 #include "localevent.h"
 #include "palette_h2.h"
@@ -197,6 +196,11 @@ void SDLFreeSurface(SDL_Surface *sf)
     }
 }
 
+SurfaceRef::SurfaceRef(const Surface & sf)
+{
+    Set(sf, true);
+}
+
 Surface::Surface() : surface(NULL)
 {
 }
@@ -213,7 +217,7 @@ Surface::Surface(u16 sw, u16 sh, bool amask) : surface(NULL)
 
 Surface::Surface(const Surface & bs) : surface(NULL)
 {
-    Set(bs);
+    Set(bs, false);
 }
 
 Surface::Surface(SDL_Surface* sf) : surface(NULL)
@@ -221,17 +225,25 @@ Surface::Surface(SDL_Surface* sf) : surface(NULL)
     Set(sf);
 }
 
+Surface::Surface(const std::string & file) : surface(NULL)
+{
+    Load(file);
+}
+
 Surface::~Surface()
 {
-    if(! isDisplay())
-	FreeSurface(*this);
+    if(! isDisplay()) FreeSurface(*this);
+}
+
+bool Surface::isDisplay(void) const
+{
+    return false;
 }
 
 /* operator = */
 Surface & Surface::operator= (const Surface & bs)
 {
-    Set(bs);
-
+    Set(bs, false);
     return *this;
 }
 
@@ -251,6 +263,11 @@ void Surface::SetDefaultDepth(u8 depth)
     }
 }
 
+Size Surface::GetSize(void) const
+{
+    return Size(w(), h());
+}
+
 u8 Surface::GetDefaultDepth(void)
 {
     return default_depth;
@@ -262,16 +279,27 @@ void Surface::Set(SDL_Surface* sf)
     surface = sf ? sf : NULL;
 }
 
-void Surface::Set(const Surface & bs)
+void Surface::Set(const Surface & bs, bool refcopy) /* copy surface */
 {
     FreeSurface(*this);
 
     if(bs.isValid())
     {
-	surface = SDL_ConvertSurface(bs.surface, bs.surface->format, bs.surface->flags);
+	if(refcopy || 1 < bs.RefCount())
+	{
+	    surface = bs.surface;
+	    if(surface) surface->refcount += 1;
+	}
+	else
+	{
+	    if(8 == bs.depth())
+		Set(bs.surface->pixels, bs.w(), bs.h(), 1, false);
+	    else
+		surface = SDL_ConvertSurface(bs.surface, bs.surface->format, bs.surface->flags);
 
-	if(!surface)
-	    Error::Except(__FUNCTION__, SDL_GetError());
+	    if(!surface)
+		Error::Except(__FUNCTION__, SDL_GetError());
+	}
     }
 }
 
@@ -334,11 +362,6 @@ bool Surface::isValid(void) const
     return surface;
 }
 
-bool Surface::isDisplay(void) const
-{
-    return surface && Display::Get().surface == surface;
-}
-
 bool Surface::Load(const char* fn)
 {
     FreeSurface(*this);
@@ -370,6 +393,11 @@ bool Surface::Save(const std::string & str) const
     return Save(str.c_str());
 }
 
+u32 Surface::RefCount(void) const
+{
+    return surface ? surface->refcount : 0;
+}
+
 u16 Surface::w(void) const
 {
     return surface ? surface->w : 0;
@@ -493,96 +521,89 @@ void Surface::SetColorKey(u32 color)
 /* draw u32 pixel */
 void Surface::SetPixel4(u16 x, u16 y, u32 color)
 {
-    if(x > surface->w || y > surface->h) return;
-    
-    u32 *bufp = static_cast<u32 *>(surface->pixels) + y * surface->pitch / 4 + x;
-
+    u32 *bufp = static_cast<u32 *>(surface->pixels) + y * (surface->pitch >> 2) + x;
     *bufp = color;
 }
 
 /* draw u24 pixel */
 void Surface::SetPixel3(u16 x, u16 y, u32 color)
 {
-    if(x > surface->w || y > surface->h) return;
     u8 *bufp = static_cast<u8 *>(surface->pixels) + y * surface->pitch + x * 3; 
-
     SetPixel24(bufp, color);
 }
 
 /* draw u16 pixel */
 void Surface::SetPixel2(u16 x, u16 y, u32 color)
 {
-    if(x > surface->w || y > surface->h) return;
-    u16 *bufp = static_cast<u16 *>(surface->pixels) + y * surface->pitch / 2 + x;
-
+    u16 *bufp = static_cast<u16 *>(surface->pixels) + y * (surface->pitch >> 1) + x;
     *bufp = static_cast<u16>(color);
 }
 
 /* draw u8 pixel */
 void Surface::SetPixel1(u16 x, u16 y, u32 color)
 {
-    if(x > surface->w || y > surface->h) return;
     u8 *bufp = static_cast<u8 *>(surface->pixels) + y * surface->pitch + x;
-
     *bufp = static_cast<u8>(color);
 }
 
 /* draw pixel */
 void Surface::SetPixel(u16 x, u16 y, u32 color)
 {
-    switch(surface->format->BytesPerPixel)
+    if(x < surface->w && y < surface->h)
     {
-	case 1:	SetPixel1(x, y, color);	break;
-	case 2:	SetPixel2(x, y, color);	break;
-	case 3:	SetPixel3(x, y, color);	break;
-	case 4:	SetPixel4(x, y, color);	break;
-	default: break;
+	switch(surface->format->BytesPerPixel)
+	{
+	    case 1:	SetPixel1(x, y, color);	break;
+	    case 2:	SetPixel2(x, y, color);	break;
+	    case 3:	SetPixel3(x, y, color);	break;
+	    case 4:	SetPixel4(x, y, color);	break;
+	    default: break;
+	}
+	if(isDisplay()) Display::Get().AddUpdateRect(x, y, 1, 1);
     }
-    if(isDisplay()) Display::Get().AddUpdateRect(x, y, 1, 1);
+    else
+	Error::Except(__FUNCTION__, "out of range");
 }
 
 u32 Surface::GetPixel4(u16 x, u16 y) const
 {
-    if(x > surface->w || y > surface->h) return 0;
     u32 *bufp = static_cast<u32 *>(surface->pixels) + y * (surface->pitch >> 2) + x;
-
     return *bufp;
 }
 
 u32 Surface::GetPixel3(u16 x, u16 y) const
 {
-    if(x > surface->w || y > surface->h) return 0;
     u8 *bufp = static_cast<u8 *>(surface->pixels) + y * surface->pitch + x * 3; 
-
     return GetPixel24(bufp);
 }
 
 u32 Surface::GetPixel2(u16 x, u16 y) const
 {
-    if(x > surface->w || y > surface->h) return 0;
     u16 *bufp = static_cast<u16 *>(surface->pixels) + y * (surface->pitch >> 1) + x;
-
     return static_cast<u32>(*bufp);
 }
 
 u32 Surface::GetPixel1(u16 x, u16 y) const
 {
-    if(x > surface->w || y > surface->h) return 0;
     u8 *bufp = static_cast<u8 *>(surface->pixels) + y * surface->pitch + x;
-
     return static_cast<u32>(*bufp);
 }
 
 u32 Surface::GetPixel(u16 x, u16 y) const
 {
-    switch(surface->format->BytesPerPixel)
+    if(x < surface->w && y < surface->h)
     {
-	case 1:	return GetPixel1(x, y);
-	case 2:	return GetPixel2(x, y);
-	case 3:	return GetPixel3(x, y);
-	case 4:	return GetPixel4(x, y);
-	default: break;
+	switch(surface->format->BytesPerPixel)
+	{
+	    case 1:	return GetPixel1(x, y);
+	    case 2:	return GetPixel2(x, y);
+	    case 3:	return GetPixel3(x, y);
+	    case 4:	return GetPixel4(x, y);
+	    default: break;
+	}
     }
+    else
+	Error::Except(__FUNCTION__, "out of range");
     
     return 0;
 }
@@ -681,7 +702,7 @@ void Surface::BlitSurface(const Surface & sf1, SDL_Rect* srt, Surface & sf2, SDL
 
         if(srt != &rt1)
 	{
-            Rect tmp = Rect::Get(Rect(*srt), Rect(rt1), true);
+            Rect tmp = Rect::Get(Rect::Get(*srt), Rect::Get(rt1), true);
 
 	    srt->x = tmp.x;
 	    srt->y = tmp.y;
@@ -691,7 +712,7 @@ void Surface::BlitSurface(const Surface & sf1, SDL_Rect* srt, Surface & sf2, SDL
 
         if(drt != &rt2)
 	{
-            Rect tmp = Rect::Get(Rect(*drt), Rect(rt2), true);
+            Rect tmp = Rect::Get(Rect::Get(*drt), Rect::Get(rt2), true);
 
 	    drt->x = tmp.x;
 	    drt->y = tmp.y;
@@ -873,8 +894,13 @@ void Surface::FreeSurface(Surface & sf)
 {
     if(sf.surface)
     {
-	SDLFreeSurface(sf.surface);
-	sf.surface = NULL;
+	if(1 < sf.surface->refcount)
+	    --sf.surface->refcount;
+	else
+	{
+	    SDLFreeSurface(sf.surface);
+	    sf.surface = NULL;
+	}
     }
 }
 
@@ -913,6 +939,7 @@ void Surface::GrayScale(void)
     if(!surface) return;
 
     u8 a, r, g, b, z;
+    a = r = g = b = z = 0;
 
     const u32 colkey = GetColorKey();
     u32 color = 0;
@@ -937,12 +964,14 @@ void Surface::Sepia(void)
 {
     if(!surface) return;
 
+    u8 r, g, b;
+    r = g = b = 0;
+
     Lock();
     for(u16 x = 0; x < surface->w; x++)
         for(u16 y = 0; y < surface->h; y++)
         {
             u32 pixel = GetPixel(x, y);
-            u8 r, g, b;
             GetRGB(pixel, &r, &g, &b);
     
             //Numbers derived from http://blogs.techrepublic.com.com/howdoi/?p=120
@@ -1003,9 +1032,10 @@ void Surface::DrawLine(u16 x1, u16 y1, u16 x2, u16 y2, u32 c)
     Unlock();
 }
 
-void Surface::MakeStencil(Surface & dst, const Surface & src, u32 col)
+Surface Surface::Stencil(const Surface & src, u32 col)
 {
-    dst.Set(src.surface->w, src.surface->h, false);
+    Surface dst;
+    dst.Set(src.surface->w,src.surface->h,src.depth(), false); // same depth for src and dst
     const u32 clkey = src.GetColorKey();
     u8 r, g, b, a;
 
@@ -1033,16 +1063,16 @@ void Surface::MakeStencil(Surface & dst, const Surface & src, u32 col)
 
     dst.Unlock();
     src.Unlock();
+
+    return dst;
 }
 
-void Surface::MakeContour(Surface & dst, const Surface & src, u32 col)
+Surface Surface::Contour(const Surface & src, u32 col)
 {
-    dst.Set(src.surface->w + 2, src.surface->h + 2, false);
-
-    Surface trf;
-    u32 fake = src.MapRGB(0x00, 0xFF, 0xFF);
-
-    MakeStencil(trf, src, fake);
+    const u32 fake = src.MapRGB(0x00, 0xFF, 0xFF);
+    Surface dst;
+    dst.Set(src.surface->w + 2,src.surface->h + 2,src.depth(), false); // same depth for src and dst
+    Surface trf = Stencil(src, fake);
     const u32 clkey = trf.GetColorKey();
 
     trf.Lock();
@@ -1071,56 +1101,93 @@ void Surface::MakeContour(Surface & dst, const Surface & src, u32 col)
 
     trf.Unlock();
     dst.Unlock();
+
+    return dst;
 }
 
-void Surface::Reflect(Surface & sf_dst, const Surface & sf_src, const u8 shape)
+Surface Surface::Reflect(const Surface & sf_src, const u8 shape)
 {
+    Surface sf_dst;
+
     if(sf_src.isValid())
     {
 	sf_dst.Set(sf_src);
-
-	const char* src = static_cast<const char*>(sf_src.surface->pixels);
-	char* dst = static_cast<char*>(sf_dst.surface->pixels);
-	const s32 size = sf_src.surface->h * sf_src.surface->pitch;
-	const u8 bpp = sf_src.surface->format->BytesPerPixel;
-
+	sf_src.Lock();
 	sf_dst.Lock();
 
 	switch(shape % 4)
 	{
     	    // normal
 	    default:
-		std::memcpy(dst, src, size);
+		// skip double copy
         	break;
-
-    	    // vertical reflect
+    
+	    // vertical reflect
     	    case 1:
-		for(s32 offset = 0; offset < size; offset += sf_src.surface->pitch)
-    		    std::memcpy(dst + size - sf_src.surface->pitch - offset,
-			src + offset, sf_src.surface->pitch);
+        	for(u16 yy = 0; yy < sf_src.h(); ++yy)
+            	    for(u16 xx = 0; xx < sf_src.w(); ++xx)
+			sf_dst.SetPixel(xx, sf_src.h() - yy - 1, sf_src.GetPixel(xx, yy));
         	break;
 
     	    // horizontal reflect
     	    case 2:
-        	for(s32 offset = 0; offset < size; offset += bpp)
-		    std::memcpy(dst + (offset % sf_src.surface->pitch) +
-			size - sf_src.surface->pitch - static_cast<s32>(offset / sf_src.surface->pitch) * sf_src.surface->pitch, src + size - offset - bpp, bpp);
+        	for(u16 yy = 0; yy < sf_src.h(); ++yy)
+            	    for(u16 xx = 0; xx < sf_src.w(); ++xx)
+			sf_dst.SetPixel(sf_src.w() - xx - 1, yy, sf_src.GetPixel(xx, yy));
 		break;
 
     	    // both variants
 	    case 3:
-        	for(s32 offset = 0; offset < size; offset += bpp)
-		    std::memcpy(dst + offset, src + size - offset - bpp, bpp);
+        	for(u16 yy = 0; yy < sf_src.h(); ++yy)
+            	    for(u16 xx = 0; xx < sf_src.w(); ++xx)
+			sf_dst.SetPixel(sf_src.w() - xx - 1, sf_src.h() - yy - 1, sf_src.GetPixel(xx, yy));
     		break;
 	}
 
+	sf_src.Unlock();
 	sf_dst.Unlock();
     }
     else
-	std::cerr << "Surface::TILReflect: " << "incorrect param" << std::endl;
+	std::cerr << __FUNCTION__ << "incorrect param" << std::endl;
+
+    return sf_dst;
 }
 
-u32 Surface::GetSize(void) const
+Surface Surface::Rotate(const Surface & sf_src, u8 parm)
+{
+    Surface sf_dst;
+
+    if(sf_src.isValid())
+    {
+	// 90 CW or 90 CCW
+	if(parm == 1 || parm == 2)
+	{
+	    sf_dst.Set(sf_src.h(), sf_src.w());
+
+	    sf_src.Lock();
+	    sf_dst.Lock();
+
+    	    for(u16 yy = 0; yy < sf_src.h(); ++yy)
+        	for(u16 xx = 0; xx < sf_src.w(); ++xx)
+		    if(parm == 1)
+			sf_dst.SetPixel(yy, sf_src.w() - xx - 1, sf_src.GetPixel(xx, yy));
+		    else
+			sf_dst.SetPixel(sf_src.h() - yy - 1, xx, sf_src.GetPixel(xx, yy));
+
+	    sf_src.Unlock();
+	    sf_dst.Unlock();
+	}
+	else
+	if(parm == 3)
+	    sf_dst = Reflect(sf_src, 3);
+    }
+    else
+	std::cerr << __FUNCTION__ << "incorrect param" << std::endl;
+
+    return sf_dst;
+}
+
+u32 Surface::GetMemoryUsage(void) const
 {
     u32 res = sizeof(surface);
 
@@ -1149,7 +1216,7 @@ std::string Surface::Info(void) const
 	    "flags" << "(" << surface->flags << ", " << (surface->flags & SDL_SRCALPHA ? "SRCALPHA" : "") << (surface->flags & SDL_SRCCOLORKEY ? "SRCCOLORKEY" : "") << "), " <<
 	    "w"<< "(" << surface->w << "), " <<
 	    "h"<< "(" << surface->h << "), " <<
-	    "size" << "(" << GetSize() << "), " <<
+	    "size" << "(" << GetMemoryUsage() << "), " <<
 	    "bpp" << "(" << static_cast<int>(surface->format->BitsPerPixel) << "), " <<
 	    "Amask" << "(" << "0x" << std::setw(8) << std::setfill('0') << std::hex << surface->format->Amask << "), " <<
 	    "colorkey" << "(" << "0x" << std::setw(8) << std::setfill('0') << surface->format->colorkey << "), " << std::dec <<
@@ -1157,7 +1224,7 @@ std::string Surface::Info(void) const
 #endif
     }
     else
-	os << "invalid";
+	os << "invalid surface";
 
     return os.str();
 }
@@ -1177,35 +1244,42 @@ u32 AVERAGE(SDL_PixelFormat* fm, u32 c1, u32 c2)
 }
 
 /* scale surface */
-void Surface::ScaleMinifyByTwo(Surface & sf_dst, const Surface & sf_src, bool event)
+Surface Surface::ScaleMinifyByTwo(const Surface & sf_src, bool event)
 {
-    if(!sf_src.isValid()) { std::cerr << __FUNCTION__ << "invalid surface" << std::endl; return; };
-    u16 x, y, x2, y2;
+    Surface sf_dst;
+    const u8 mul = 2;
 
-    u8 mul = 2;
-    u16 w = sf_src.w() / mul;
-    u16 h = sf_src.h() / mul;
-
-    if(2 > w || 2 > h){ std::cerr << __FUNCTION__ << "small size" << std::endl; return; };
-
-    sf_dst.Set(w, h, sf_src.depth(), sf_src.amask());
-
-    sf_dst.Lock();
-    sf_src.Lock();
-    for(y = 0; y < h; y++)
+    if(sf_src.isValid() && 4 < sf_src.w() && 4 < sf_src.h())
     {
-       y2 = mul * y;
-       for(x = 0; x < w; x++)
-       {
-	    x2 = mul * x;
-	    const u32 & p = AVERAGE(sf_src.surface->format, sf_src.GetPixel(x2, y2), sf_src.GetPixel(x2 + 1, y2));
-	    const u32 & q = AVERAGE(sf_src.surface->format, sf_src.GetPixel(x2, y2 + 1), sf_src.GetPixel(x2 + 1, y2 + 1));
-	    sf_dst.SetPixel(x, y, AVERAGE(sf_src.surface->format, p, q));
-	    if(event) LocalEvent::Get().HandleEvents(false);
-       }
+	u16 x, y, x2, y2;
+
+	u16 w = sf_src.w() / mul;
+	u16 h = sf_src.h() / mul;
+
+	sf_dst.Set(w, h, sf_src.depth(), sf_src.amask());
+
+	sf_dst.Lock();
+	sf_src.Lock();
+
+	for(y = 0; y < h; y++)
+	{
+    	    y2 = mul * y;
+	    for(x = 0; x < w; x++)
+    	    {
+		x2 = mul * x;
+		const u32 & p = AVERAGE(sf_src.surface->format, sf_src.GetPixel(x2, y2), sf_src.GetPixel(x2 + 1, y2));
+		const u32 & q = AVERAGE(sf_src.surface->format, sf_src.GetPixel(x2, y2 + 1), sf_src.GetPixel(x2 + 1, y2 + 1));
+		sf_dst.SetPixel(x, y, AVERAGE(sf_src.surface->format, p, q));
+		if(event) LocalEvent::Get().HandleEvents(false);
+    	    }
+	}
+	sf_src.Unlock();
+	sf_dst.Unlock();
     }
-    sf_src.Unlock();
-    sf_dst.Unlock();
+    else
+	std::cerr << __FUNCTION__ << "invalid surface" << std::endl;
+
+    return sf_dst;
 }
 
 void Surface::Swap(Surface & sf1, Surface & sf2)
