@@ -1,25 +1,26 @@
-/*************************************************************************** 
+/***************************************************************************
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
  *                                                                         *
  *   Part of the Free Heroes2 Engine:                                      *
  *   http://sourceforge.net/projects/fheroes2                              *
- *                                                                         * 
- *   This program is free software; you can redistribute it and/or modify  * 
- *   it under the terms of the GNU General Public License as published by  * 
- *   the Free Software Foundation; either version 2 of the License, or     * 
- *   (at your option) any later version.                                   * 
- *                                                                         * 
- *   This program is distributed in the hope that it will be useful,       * 
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
- *   GNU General Public License for more details.                          * 
- *                                                                         * 
- *   You should have received a copy of the GNU General Public License     * 
- *   along with this program; if not, write to the                         * 
- *   Free Software Foundation, Inc.,                                       * 
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
+#include <sstream>
 #include <cstring>
 #include <algorithm>
 #include "gamedefs.h"
@@ -27,15 +28,13 @@
 #include "agg.h"
 #include "cursor.h"
 #include "dialog.h"
-#include "editor_dialogs.h"
+#include "dialog_selectitems.h"
 #include "heroes.h"
 #include "settings.h"
+#include "skill_static.h"
+#include "game_static.h"
 #include "skill.h"
 
-#ifdef WITH_XML
-#include "xmlccwrap.h"
-#endif
-
 namespace Skill
 {
     u8 SecondaryGetWeightSkillFromRace(u8 race, u8 skill);
@@ -43,94 +42,6 @@ namespace Skill
     std::vector<u8> SecondarySkills(void);
 }
 
-struct level_t
-{
-    u16 basic;
-    u16 advanced;
-    u16 expert;
-};
-
-struct primary_t
-{
-    u8 attack;
-    u8 defense;
-    u8 power;
-    u8 knowledge;
-};
-
-struct secondary_t
-{
-    u8 archery;
-    u8 ballistics;
-    u8 diplomacy;
-    u8 eagleeye;
-    u8 estates;
-    u8 leadership;
-    u8 logistics;
-    u8 luck;
-    u8 mysticism;
-    u8 navigation;
-    u8 necromancy;
-    u8 pathfinding;
-    u8 scouting;
-    u8 wisdom;
-};
-
-struct skillstats_t
-{
-    const char* id;
-    primary_t   captain_primary;
-    primary_t   initial_primary;
-    u8          initial_book;
-    u8          initial_spell;
-    secondary_t initial_secondary;
-    u8		over_level;
-    primary_t   mature_primary_under;
-    primary_t   mature_primary_over;
-    secondary_t mature_secondary;
-};
-
-struct skillvalues_t
-{
-    const char *id;
-    level_t values;
-};
-
-skillstats_t _skillstats[] = {
-    { "knight",      { 1, 1, 1, 1 }, { 2, 2, 1, 1 }, 0, 0, { 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 }, 10, {35,45,10,10 }, {25,25,25,25 }, { 2, 4, 3, 1, 3, 5, 3, 1, 1, 2, 0, 3, 2, 2 } },
-    { "barbarian",   { 1, 1, 1, 1 }, { 3, 1, 1, 1 }, 0, 0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0 }, 10, {55,35, 5, 5 }, {30,30,20,20 }, { 3, 3, 2, 1, 2, 3, 3, 2, 1, 3, 0, 4, 4, 1 } },
-    { "sorceress",   { 0, 0, 2, 2 }, { 0, 0, 2, 3 }, 1,15, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1 }, 10, {10,10,30,50 }, {20,20,30,30 }, { 3, 3, 2, 2, 2, 1, 2, 3, 3, 4, 0, 2, 1, 4 } },
-    { "warlock",     { 0, 0, 2, 2 }, { 0, 0, 3, 2 }, 1,19, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1 }, 10, {10,10,50,30 }, {20,20,30,30 }, { 1, 3, 2, 3, 2, 1, 2, 1, 3, 2, 1, 2, 4, 5 } },
-    { "wizard",      { 0, 0, 2, 2 }, { 0, 1, 2, 2 }, 1,17, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2 }, 10, {10,10,40,40 }, {20,20,30,30 }, { 1, 3, 2, 3, 2, 2, 2, 2, 4, 2, 0, 2, 2, 5 } },
-    { "necromancer", { 0, 0, 2, 2 }, { 1, 0, 2, 2 }, 1,10, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1 }, 10, {15,15,35,35 }, {25,25,25,25 }, { 1, 3, 2, 3, 2, 0, 2, 1, 3, 2, 5, 3, 1, 4 } },
-    { NULL,          { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, 0, 0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, 10, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } }
-};
-
-skillvalues_t _skillvalues[] = {
-    { "pathfinding", { 25, 50,100} },
-    { "archery",     { 10, 25, 50} },
-    { "logistics",   { 10, 20, 30} },
-    { "scouting",    {  1,  2,  3} },
-    { "diplomacy",   { 25, 50,100} },
-    { "navigation",  { 33, 66,100} },
-    { "leadership",  {  1,  2,  3} },
-    { "wisdom",      {  3,  4,  5} },
-    { "mysticism",   {  2,  3,  4} },
-    { "luck",        {  1,  2,  3} },
-    { "ballistics",  {  0,  0,  0} },
-    { "eagleeye",    { 20, 30, 40} },
-    { "necromancy",  { 10, 20, 30} },
-    { "estates",     {100,250,500} },
-    { NULL,          {  0,  0,  0} },
-};
-
-secondary_t _from_witchs_hut = {
-    /* archery */   1, /* ballistics */ 1, /* diplomacy */ 1, /* eagleeye */    1,
-    /* estates */   1, /* leadership */ 0, /* logistics */ 1, /* luck */        1,
-    /* mysticism */ 1, /* navigation */ 1, /* necromancy*/ 0, /* pathfinding */ 1,
-    /* scouting */  1, /* wisdom */ 1
-};
-
 /*
 struct defaulthero_t
 {
@@ -147,151 +58,19 @@ struct defaulthero_t
 };
 */
 
-const skillstats_t* GetSkillStats(u8 race)
-{
-    const char* id = NULL;
-
-    switch(race)
-    {
-	case Race::KNGT:	id = "knight"; break;
-	case Race::BARB:	id = "barbarian"; break;
-	case Race::SORC:	id = "sorceress"; break;
-	case Race::WRLK:	id = "warlock"; break;
-	case Race::WZRD:	id = "wizard"; break;
-	case Race::NECR:	id = "necromancer"; break;
-	default:
-	    DEBUG(DBG_GAME, DBG_WARN, "is NULL"); break;
-    }
-
-    const skillstats_t* ptr = &_skillstats[0];
-    while(ptr->id && id && std::strcmp(id, ptr->id)) ++ptr;
-
-    return id ? ptr : NULL;
-}
-
-#ifdef WITH_XML
-void LoadPrimarySection(const TiXmlElement* xml, primary_t & skill)
-{
-    if(!xml) return;
-    int value;
-    xml->Attribute("attack", &value);    skill.attack = value;
-    xml->Attribute("defense", &value);   skill.defense = value;
-    xml->Attribute("power", &value);     skill.power = value;
-    xml->Attribute("knowledge", &value); skill.knowledge = value;
-}
-
-void LoadSecondarySection(const TiXmlElement* xml, secondary_t & sec)
-{
-    if(!xml) return;
-    int value;
-    xml->Attribute("archery", &value);     sec.archery = value;
-    xml->Attribute("ballistics", &value);  sec.ballistics = value;
-    xml->Attribute("diplomacy", &value);   sec.diplomacy = value;
-    xml->Attribute("eagleeye", &value);    sec.eagleeye = value;
-    xml->Attribute("estates", &value);     sec.estates = value;
-    xml->Attribute("leadership", &value);  sec.leadership = value;
-    xml->Attribute("logistics", &value);   sec.logistics = value;
-    xml->Attribute("luck", &value);        sec.luck = value;
-    xml->Attribute("mysticism", &value);   sec.mysticism = value;
-    xml->Attribute("navigation", &value);  sec.navigation = value;
-    xml->Attribute("necromancy", &value);  sec.necromancy = value;
-    xml->Attribute("pathfinding", &value); sec.pathfinding = value;
-    xml->Attribute("scouting", &value);    sec.scouting = value;
-    xml->Attribute("wisdom", &value);      sec.wisdom = value;
-}
-#endif
-
-void Skill::UpdateStats(const std::string & spec)
-{
-#ifdef WITH_XML
-    // parse skills.xml
-    TiXmlDocument doc;
-    const TiXmlElement* xml_skills = NULL;
-                
-    if(doc.LoadFile(spec.c_str()) &&
-	NULL != (xml_skills = doc.FirstChildElement("skills")))
-    {
-	const TiXmlElement* xml_captain = xml_skills->FirstChildElement("captain");
-	const TiXmlElement* xml_initial = xml_skills->FirstChildElement("initial");
-	const TiXmlElement* xml_maturity = xml_skills->FirstChildElement("maturity");
-	const TiXmlElement* xml_secondary = xml_maturity ? xml_maturity->FirstChildElement("secondary") : NULL;
-	const TiXmlElement* xml_primary = xml_maturity ? xml_maturity->FirstChildElement("primary") : NULL;
-	const TiXmlElement* xml_under = xml_primary ? xml_primary->FirstChildElement("under") : NULL;
-	const TiXmlElement* xml_over = xml_primary ? xml_primary->FirstChildElement("over") : NULL;
-	skillstats_t *ptr = &_skillstats[0];
-	int value;
-
-	while(ptr->id)
-	{
-	    const TiXmlElement* initial_race = xml_initial ? xml_initial->FirstChildElement(ptr->id) : NULL;
-
-	    if(initial_race)
-	    {
-		LoadPrimarySection(initial_race, ptr->initial_primary);
-		LoadSecondarySection(initial_race, ptr->initial_secondary);
-
-		initial_race->Attribute("book", &value);  ptr->initial_book = value;
-		initial_race->Attribute("spell", &value); ptr->initial_spell = value;
-	    }
-
-	    const TiXmlElement* captain_race = xml_captain ? xml_captain->FirstChildElement(ptr->id) : NULL;
-	    if(captain_race) LoadPrimarySection(captain_race, ptr->captain_primary);
-
-	    const TiXmlElement* under_race = xml_under ? xml_under->FirstChildElement(ptr->id) : NULL;
-	    if(under_race) LoadPrimarySection(under_race, ptr->mature_primary_under);
-
-	    const TiXmlElement* over_race = xml_over ? xml_over->FirstChildElement(ptr->id) : NULL;
-	    if(over_race)
-	    {
-		LoadPrimarySection(over_race, ptr->mature_primary_over);
-		over_race->Attribute("level", &value);
-		if(value) ptr->over_level = value;
-	    }
-
-	    const TiXmlElement* secondary_race = xml_secondary ? xml_secondary->FirstChildElement(ptr->id) : NULL;
-	    if(secondary_race) LoadSecondarySection(secondary_race, ptr->mature_secondary);
-
-	    ++ptr;
-	}
-
-	xml_secondary = xml_skills->FirstChildElement("secondary");
-	if(xml_secondary)
-	{
-	    skillvalues_t* ptr2 = &_skillvalues[0];
-
-	    while(ptr2->id)
-	    {
-		const TiXmlElement* xml_sec = xml_secondary->FirstChildElement(ptr2->id);
-
-		if(xml_sec)
-		{
-		    xml_sec->Attribute("basic", &value); ptr2->values.basic = value;
-		    xml_sec->Attribute("advanced", &value); ptr2->values.advanced = value;
-		    xml_sec->Attribute("expert", &value); ptr2->values.expert = value;
-		}
-
-		++ptr2;
-	    }
-	}
-
-	const TiXmlElement* xml_witchs_hut = xml_skills->FirstChildElement("witchs_hut");
-	if(xml_witchs_hut)
-	    LoadSecondarySection(xml_witchs_hut, _from_witchs_hut);
-    }
-    else
-    VERBOSE(spec << ": " << doc.ErrorDesc());
-#endif
-}
-
 u16 Skill::Secondary::GetValues(void) const
 {
+    const values_t* val = GameStatic::GetSkillValues(Skill());
+
+    if(val)
     switch(Level())
     {
-	case Level::BASIC:	return _skillvalues[Skill() - 1].values.basic;
-	case Level::ADVANCED:	return _skillvalues[Skill() - 1].values.advanced;
-	case Level::EXPERT:	return _skillvalues[Skill() - 1].values.expert;
-	default: break;
+        case Level::BASIC:      return val->values.basic;
+        case Level::ADVANCED:   return val->values.advanced;
+        case Level::EXPERT:     return val->values.expert;
+        default: break;
     }
+
     return 0;
 }
 
@@ -299,25 +78,25 @@ Skill::Primary::Primary() : attack(0), defense(0), power(0), knowledge(0)
 {
 }
 
-void Skill::Primary::LoadDefaults(u8 type, u8 race, Primary & skill)
+void Skill::Primary::LoadDefaults(u8 type, u8 race)
 {
-    const skillstats_t* ptr = GetSkillStats(race);
+    const stats_t* ptr = GameStatic::GetSkillStats(race);
 
     if(ptr)
     switch(type)
     {
 	case CAPTAIN:
-	    skill.attack = ptr->captain_primary.attack;
-	    skill.defense = ptr->captain_primary.defense;
-	    skill.power = ptr->captain_primary.power;
-	    skill.knowledge = ptr->captain_primary.knowledge;
+	    attack = ptr->captain_primary.attack;
+	    defense = ptr->captain_primary.defense;
+	    power = ptr->captain_primary.power;
+	    knowledge = ptr->captain_primary.knowledge;
 	    break;
 
 	case HEROES:
-	    skill.attack = ptr->initial_primary.attack;
-	    skill.defense = ptr->initial_primary.defense;
-	    skill.power = ptr->initial_primary.power;
-	    skill.knowledge = ptr->initial_primary.knowledge;
+	    attack = ptr->initial_primary.attack;
+	    defense = ptr->initial_primary.defense;
+	    power = ptr->initial_primary.power;
+	    knowledge = ptr->initial_primary.knowledge;
 	    break;
 
 	default:
@@ -327,7 +106,7 @@ void Skill::Primary::LoadDefaults(u8 type, u8 race, Primary & skill)
 
 u8 Skill::Primary::GetInitialSpell(u8 race)
 {
-    const skillstats_t* ptr = GetSkillStats(race);
+    const stats_t* ptr = GameStatic::GetSkillStats(race);
     return ptr ? ptr->initial_spell : 0;
 }
 
@@ -335,7 +114,7 @@ u8 Skill::Primary::LevelUp(u8 race, u8 level)
 {
     Rand::Queue percents(MAXPRIMARYSKILL);
 
-    const skillstats_t* ptr = GetSkillStats(race);
+    const stats_t* ptr = GameStatic::GetSkillStats(race);
     if(ptr)
     {
 	if(ptr->over_level > level)
@@ -384,6 +163,53 @@ const char* Skill::Primary::String(u8 skill)
     return str_skill[4];
 }
 
+std::string Skill::Primary::StringDescription(u8 skill, const Heroes* hero)
+{
+    std::string res, ext;
+
+    switch(skill)
+    {
+	case ATTACK:
+	    res = _("Your attack skill is a bonus added to each creature's attack skill.");
+	    if(hero) hero->GetAttack(&ext);
+	    break;
+
+        case DEFENSE:
+	    res = _("Your defense skill is a bonus added to each creature's defense skill.");
+	    if(hero) hero->GetDefense(&ext);
+	    break;
+
+        case POWER:
+	    res = _("Your spell power determines the length or power of a spell.");
+	    if(hero) hero->GetPower(&ext);
+	    break;
+
+        case KNOWLEDGE:
+	    res = _("Your knowledge determines how many spell points your hero may have. Under normal cirumstances, a hero is limited to 10 spell points per level of knowledge.");
+	    if(hero) hero->GetKnowledge(&ext);
+	    break;
+
+        default: break;
+    }
+
+    if(ext.size())
+    {
+        res.append("\n \n");
+        res.append(_("Current Modifiers:"));
+        res.append("\n \n");
+        res.append(ext);
+    }
+
+    return res;
+}
+
+std::string Skill::Primary::StringSkills(const std::string & sep) const
+{
+    std::ostringstream os;
+    os << GetString(attack) << sep << GetString(defense) << sep << GetString(knowledge) << sep << GetString(power);
+    return os.str();
+}
+
 const char* Skill::Level::String(u8 level)
 {
     const char* str_level[] = { "None", _("skill|Basic"), _("skill|Advanced"), _("skill|Expert") };
@@ -395,7 +221,7 @@ const char* Skill::Level::String(u8 level)
 	case EXPERT:	return str_level[3];
 	default: break;
     }
-    
+
     return str_level[0];
 }
 
@@ -469,23 +295,28 @@ bool Skill::Secondary::isValid(void) const
 
 u8 Skill::Secondary::RandForWitchsHut(void)
 {
+    const Skill::secondary_t* sec = GameStatic::GetSkillForWitchsHut();
     std::vector<u8> v;
-    v.reserve(14);
 
-    if(_from_witchs_hut.archery)	v.push_back(ARCHERY);
-    if(_from_witchs_hut.ballistics)	v.push_back(BALLISTICS);
-    if(_from_witchs_hut.diplomacy)	v.push_back(DIPLOMACY);
-    if(_from_witchs_hut.eagleeye)	v.push_back(EAGLEEYE);
-    if(_from_witchs_hut.estates)	v.push_back(ESTATES);
-    if(_from_witchs_hut.leadership)	v.push_back(LEADERSHIP);
-    if(_from_witchs_hut.logistics)	v.push_back(LOGISTICS);
-    if(_from_witchs_hut.luck)		v.push_back(LUCK);
-    if(_from_witchs_hut.mysticism)	v.push_back(MYSTICISM);
-    if(_from_witchs_hut.navigation)	v.push_back(NAVIGATION);
-    if(_from_witchs_hut.necromancy)	v.push_back(NECROMANCY);
-    if(_from_witchs_hut.pathfinding)	v.push_back(PATHFINDING);
-    if(_from_witchs_hut.scouting)	v.push_back(SCOUTING);
-    if(_from_witchs_hut.wisdom)		v.push_back(WISDOM);
+    if(sec)
+    {
+	v.reserve(14);
+
+	if(sec->archery)        v.push_back(ARCHERY);
+	if(sec->ballistics)     v.push_back(BALLISTICS);
+	if(sec->diplomacy)      v.push_back(DIPLOMACY);
+	if(sec->eagleeye)       v.push_back(EAGLEEYE);
+	if(sec->estates)        v.push_back(ESTATES);
+	if(sec->leadership)     v.push_back(LEADERSHIP);
+	if(sec->logistics)      v.push_back(LOGISTICS);
+	if(sec->luck)           v.push_back(LUCK);
+	if(sec->mysticism)      v.push_back(MYSTICISM);
+	if(sec->navigation)     v.push_back(NAVIGATION);
+	if(sec->necromancy)     v.push_back(NECROMANCY);
+	if(sec->pathfinding)    v.push_back(PATHFINDING);
+	if(sec->scouting)       v.push_back(SCOUTING);
+	if(sec->wisdom)         v.push_back(WISDOM);
+    }
 
     return v.empty() ? UNKNOWN : *Rand::Get(v);
 }
@@ -504,7 +335,7 @@ u8 Skill::Secondary::GetIndexSprite2(void) const
 
 const char* Skill::Secondary::String(u8 skill)
 {
-    const char* str_skill[] = { _("Pathfinding"), _("Archery"), _("Logistics"), _("Scouting"), _("Diplomacy"), _("Navigation"), 
+    const char* str_skill[] = { _("Pathfinding"), _("Archery"), _("Logistics"), _("Scouting"), _("Diplomacy"), _("Navigation"),
 	_("Leadership"), _("Wisdom"), _("Mysticism"), _("Luck"), _("Ballistics"), _("Eagle Eye"), _("Necromancy"), _("Estates"), "Unknown"  };
 
     switch(skill)
@@ -523,7 +354,7 @@ const char* Skill::Secondary::String(u8 skill)
         case EAGLEEYE:		return str_skill[11];
         case NECROMANCY:	return str_skill[12];
 	case ESTATES:		return str_skill[13];
-	
+
 	default: break;
     }
 
@@ -561,18 +392,18 @@ std::string Skill::Secondary::GetDescription(void) const
     switch(Skill())
     {
 	case PATHFINDING:
-	    str = ngettext("Reduces the movement penalty for rough terrain by %{count} percent.", 
+	    str = ngettext("Reduces the movement penalty for rough terrain by %{count} percent.",
 			"Reduces the movement penalty for rough terrain by %{count} percent.", count); break;
-        case ARCHERY:	
+        case ARCHERY:
 	    str = ngettext("Increases the damage done by range attacking creatures by %{count} percent.",
 			"Increases the damage done by range attacking creatures by %{count} percent.", count); break;
-        case LOGISTICS:	
+        case LOGISTICS:
 	    str = ngettext("Increases your hero's movement points by %{count} percent.",
 			"Increases your hero's movement points by %{count} percent.", count); break;
-        case SCOUTING:	
+        case SCOUTING:
 	    str = ngettext("Increases your hero's viewable area by %{count} square.",
 			"Increases your hero's viewable area by %{count} squares.", count); break;
-        case DIPLOMACY:	
+        case DIPLOMACY:
 	    str = _("Allows you to negotiate with monsters who are weaker than your group.");
 	    str.append(" ");
 	    str.append(ngettext("Approximately %{count} percent of the creatures may offer to join you.",
@@ -581,9 +412,9 @@ std::string Skill::Secondary::GetDescription(void) const
 	    str = ngettext("Increases your hero's movement points over water by %{count} percent.",
 		    "Increases your hero's movement points over water by %{count} percent.", count); break;
         case LEADERSHIP:
-	    str = ngettext("Increases your hero's troops' morale by %{count}.",
-		    "Increases your hero's troops' morale by %{count}.", count); break;
-        case WISDOM:	
+	    str = ngettext("Increases your hero's troops morale by %{count}.",
+		    "Increases your hero's troops morale by %{count}.", count); break;
+        case WISDOM:
 	{
 	    switch(Level())
 	    {
@@ -597,10 +428,10 @@ std::string Skill::Secondary::GetDescription(void) const
 	    }
 	    break;
 	}
-        case MYSTICISM:	
+        case MYSTICISM:
 	    str = ngettext("Regenerates %{count} of your hero's spell point per day.",
 		    "Regenerates %{count} of your hero's spell points per day.", count); break;
-        case LUCK:	
+        case LUCK:
 	    str = ngettext("Increases your hero's luck by %{count}.",
 		    "Increases your hero's luck by %{count}.", count); break;
         case BALLISTICS:
@@ -615,7 +446,7 @@ std::string Skill::Secondary::GetDescription(void) const
 		default: break;
 	    }
 	    break;
-        case EAGLEEYE:	
+        case EAGLEEYE:
 	    switch(Level())
 	    {
 		case Level::BASIC:
@@ -633,13 +464,13 @@ std::string Skill::Secondary::GetDescription(void) const
         case NECROMANCY:
 	    str = ngettext("Allows %{count} percent of the creatures killed in combat to be brought back from the dead as Skeletons.",
 		    "Allows %{count} percent of the creatures killed in combat to be brought back from the dead as Skeletons.", count); break;
-	case ESTATES:	
+	case ESTATES:
 	    str = ngettext("Your hero produce %{count} gold pieces per turn as tax revenue from estates.",
 		    "Your hero produces %{count} gold pieces per turn as tax revenue from estates.", count); break;
 	default: break;
     }
 
-    String::Replace(str, "%{count}", count);
+    StringReplace(str, "%{count}", count);
 
     return str;
 }
@@ -655,7 +486,7 @@ Skill::SecSkills::SecSkills(u8 race)
 
     if(race & Race::ALL)
     {
-	const skillstats_t* ptr = GetSkillStats(race);
+	const stats_t* ptr = GameStatic::GetSkillStats(race);
 
 	if(ptr)
 	{
@@ -731,9 +562,15 @@ std::string Skill::SecSkills::String(void) const
     return os.str();
 }
 
+void Skill::SecSkills::FillMax(const Skill::Secondary & skill)
+{
+    if(size() < HEROESMAXSKILL)
+	resize(HEROESMAXSKILL, skill);
+}
+
 u8 Skill::SecondaryGetWeightSkillFromRace(u8 race, u8 skill)
 {
-    const skillstats_t* ptr = GetSkillStats(race);
+    const stats_t* ptr = GameStatic::GetSkillStats(race);
 
     if(ptr)
     {
@@ -842,176 +679,6 @@ void Skill::SecSkills::FindSkillsForLevelUp(u8 race, Secondary & sec1, Secondary
     }
 }
 
-
-
-
-
-
-
-SecondarySkillBar::SecondarySkillBar() : skills(NULL), use_mini_sprite(false), can_change(false)
-{
-}
-
-const Rect & SecondarySkillBar::GetArea(void) const
-{
-    return pos;
-}
-
-void SecondarySkillBar::SetSkills(Skill::SecSkills & v)
-{
-    skills = &v;
-    CalcSize();
-}
-
-void SecondarySkillBar::SetUseMiniSprite(void)
-{
-    use_mini_sprite = true;
-}
-
-void SecondarySkillBar::SetInterval(u8 i)
-{
-    interval = i;
-    CalcSize();
-}
-
-void SecondarySkillBar::SetPos(s16 sx, s16 sy)
-{
-    pos.x = sx;
-    pos.y = sy;
-    CalcSize();
-}
-
-void SecondarySkillBar::SetChangeMode(void)
-{
-    can_change = true;
-}
-
-void SecondarySkillBar::CalcSize(void)
-{
-    pos.w = 0;
-    pos.h = 0;
-
-    if(skills)
-    {
-	const Sprite & sprite = AGG::GetICN((use_mini_sprite ? ICN::MINISS : ICN::SECSKILL), 0);
-	pos.h = sprite.h();
-	pos.w = HEROESMAXSKILL * (sprite.w() + interval);
-    }
-}
-
-void SecondarySkillBar::Redraw(void)
-{
-    Point dst_pt(pos);
-    Text text;
-    text.Set(Font::SMALL);
-
-    for(u8 ii = 0; ii < HEROESMAXSKILL; ++ii)
-    {
-        const Skill::Secondary & skill = ii < skills->size() ? skills->at(ii) : Skill::Secondary();
-
-        if(skill.isValid())
-        {
-            const Sprite & sprite_skill = AGG::GetICN((use_mini_sprite ? ICN::MINISS : ICN::SECSKILL), (use_mini_sprite ? skill.GetIndexSprite2() : skill.GetIndexSprite1()));
-            sprite_skill.Blit(dst_pt);
-
-            if(use_mini_sprite)
-	    {
-        	text.Set(GetString(skill.Level()));
-        	text.Blit(dst_pt.x + (sprite_skill.w() - text.w()) - 3, dst_pt.y + sprite_skill.h() - 12);
-	    }
-	    else
-	    {
-        	text.Set(Skill::Secondary::String(skill.Skill()));
-        	text.Blit(dst_pt.x + (sprite_skill.w() - text.w()) / 2, dst_pt.y + 3);
-
-        	text.Set(Skill::Level::String(skill.Level()));
-        	text.Blit(dst_pt.x + (sprite_skill.w() - text.w()) / 2, dst_pt.y + 50);
-	    }
-
-    	    dst_pt.x += (use_mini_sprite ? 32 : sprite_skill.w()) + interval;
-        }
-	else
-	{
-            const Sprite & sprite_skill = AGG::GetICN((use_mini_sprite ? ICN::HSICONS : ICN::SECSKILL), 0);
-
-	    if(use_mini_sprite)
-        	sprite_skill.Blit(Rect((sprite_skill.w() - 32) / 2, 20, 32, 32), dst_pt);
-	    else
-        	sprite_skill.Blit(dst_pt);
-
-    	    dst_pt.x += (use_mini_sprite ? 32 : sprite_skill.w()) + interval;
-	}
-    }
-}
-
-u8 SecondarySkillBar::GetIndexFromCoord(const Point & cu)
-{
-    const Sprite & sprite_skill = AGG::GetICN((use_mini_sprite ? ICN::MINISS : ICN::SECSKILL), 0);
-    return (pos & cu) ? (cu.x - pos.x) / (sprite_skill.w() + interval) : 0;
-}
-
-bool SecondarySkillBar::QueueEventProcessing(void)
-{
-    Display & display = Display::Get();
-    Cursor & cursor = Cursor::Get();
-    LocalEvent & le = LocalEvent::Get();
-    const Point & cu = le.GetMouseCursor();
-    bool modify = false;
-
-    if(!(pos & cu) || !skills) return false;
-
-    u8 ii = GetIndexFromCoord(cu);
-    const Sprite & sprite_skill = AGG::GetICN((use_mini_sprite ? ICN::MINISS : ICN::SECSKILL), 0);
-    const Rect tile(pos.x + (ii * (sprite_skill.w() + interval)), pos.y, sprite_skill.w(), sprite_skill.h());
-
-    if(ii < skills->size())
-    {
-	Skill::Secondary & skill = skills->at(ii);
-
-	if(skill.isValid())
-	{
-	    if(le.MouseClickLeft(tile))
-    	    {
-        	cursor.Hide();
-        	Dialog::SecondarySkillInfo(skill, true);
-        	cursor.Show();
-        	display.Flip();
-	    }
-	    else
-	    if(le.MousePressRight(tile))
-	    {
-		if(can_change)
-		{
-		    skill.Reset();
-		    modify = true;
-		}
-		else
-		{
-		    cursor.Hide();
-		    Dialog::SecondarySkillInfo(skill, false);
-		    cursor.Show();
-		    display.Flip();
-		}
-	    }
-	}
-    }
-    else
-    if(ii < MAXSECONDARYSKILL)
-    {
-	if(can_change && le.MouseClickLeft(tile))
-	{
-	    Skill::Secondary alt = Dialog::SelectSecondarySkill();
-	    if(alt.isValid() && skills)
-	    {
-		skills->AddSkill(alt);
-		modify = true;
-	    }
-	}
-    }
-
-    return modify;
-}
-
 void StringAppendModifiers(std::string & str, s8 value)
 {
     if(value < 0) str.append(" "); // '-' present
@@ -1048,3 +715,270 @@ s8 Skill::GetLuckModifiers(u8 level, std::string* strs = NULL)
 
     return skill.GetValues();
 }
+
+StreamBase & Skill::operator<< (StreamBase & msg, const Primary & skill)
+{
+    return msg << skill.attack << skill.defense << skill.knowledge << skill.power;
+}
+
+StreamBase & Skill::operator>> (StreamBase & msg, Primary & skill)
+{
+    return msg >> skill.attack >> skill.defense >> skill.knowledge >> skill.power;
+}
+
+PrimarySkillsBar::PrimarySkillsBar(const Heroes* hr, bool mini) : hero(hr), use_mini_sprite(mini), toff(0, 0)
+{
+    content.push_back(Skill::Primary::ATTACK);
+    content.push_back(Skill::Primary::DEFENSE);
+    content.push_back(Skill::Primary::POWER);
+    content.push_back(Skill::Primary::KNOWLEDGE);
+
+    if(use_mini_sprite)
+    {
+	const Sprite & sprite = AGG::GetICN(ICN::HSICONS, 0);
+	const Rect rt(25, 20, 34, 34);
+	backsf.Set(rt.w, rt.h);
+	sprite.Blit(rt, 0, 0, backsf);
+	Cursor::DrawCursor(backsf, 0x70, true);
+	SetItemSize(rt.w, rt.h);
+    }
+    else
+    {
+	const Sprite & sprite = AGG::GetICN(ICN::PRIMSKIL, 0);
+	SetItemSize(sprite.w(), sprite.h());
+    }
+
+    SetContent(content);
+}
+
+void PrimarySkillsBar::SetTextOff(s16 ox, s16 oy)
+{
+    toff = Point(ox, oy);
+}
+
+void PrimarySkillsBar::RedrawBackground(const Rect & pos, Surface & dstsf)
+{
+    if(use_mini_sprite)
+	backsf.Blit(pos, dstsf);
+}
+
+void PrimarySkillsBar::RedrawItem(Skill::Primary::skill_t & skill, const Rect & pos, Surface & dstsf)
+{
+    if(Skill::Primary::UNKNOWN != skill)
+    {
+	if(use_mini_sprite)
+	{
+	    const Sprite & backSprite = AGG::GetICN(ICN::SWAPWIN, 0);
+	    const u8 ww = 32;
+	    Text text("", Font::SMALL);
+	    const Point dstpt(pos.x + (pos.w - ww) / 2, pos.y + (pos.h - ww) / 2);
+
+	    switch(skill)
+	    {
+    		case Skill::Primary::ATTACK:
+		    backSprite.Blit(Rect(217, 52, ww, ww), dstpt, dstsf);
+		    if(hero) text.Set(GetString(hero->GetAttack()));
+		    break;
+
+    		case Skill::Primary::DEFENSE:
+		    backSprite.Blit(Rect(217, 85, ww, ww), dstpt, dstsf);
+		    if(hero) text.Set(GetString(hero->GetDefense()));
+		    break;
+
+    		case Skill::Primary::POWER:
+		    backSprite.Blit(Rect(217, 118, ww, ww), dstpt, dstsf);
+		    if(hero) text.Set(GetString(hero->GetPower()));
+		    break;
+
+    		case Skill::Primary::KNOWLEDGE:
+		    backSprite.Blit(Rect(217, 151, ww, ww), dstpt, dstsf);
+		    if(hero) text.Set(GetString(hero->GetKnowledge()));
+		    break;
+
+		    default: break;
+	    }
+
+	    if(hero) text.Blit(pos.x + (pos.w + toff.x - text.w()) / 2, pos.y + pos.h + toff.y, dstsf);
+	}
+	else
+	{
+    	    const Sprite & sprite = AGG::GetICN(ICN::PRIMSKIL, skill - 1);
+	    sprite.Blit(pos.x + (pos.w - sprite.w()) / 2, pos.y + (pos.h - sprite.h()) / 2, dstsf);
+
+    	    Text text(Skill::Primary::String(skill), Font::SMALL);
+    	    text.Blit(pos.x + (pos.w - text.w()) / 2, pos.y + 3, dstsf);
+
+	    if(hero)
+	    {
+		switch(skill)
+		{
+    		    case Skill::Primary::ATTACK:
+			text.Set(GetString(hero->GetAttack()), Font::BIG);
+			break;
+
+    		    case Skill::Primary::DEFENSE:
+			text.Set(GetString(hero->GetDefense()), Font::BIG);
+			break;
+
+    		    case Skill::Primary::POWER:
+			text.Set(GetString(hero->GetPower()), Font::BIG);
+			break;
+
+    		    case Skill::Primary::KNOWLEDGE:
+			text.Set(GetString(hero->GetKnowledge()), Font::BIG);
+			break;
+
+		    default: break;
+		}
+
+		text.Blit(pos.x + (pos.w - text.w()) / 2, pos.y + pos.h - text.h() - 3, dstsf);
+	    }
+	}
+    }
+}
+
+bool PrimarySkillsBar::ActionBarSingleClick(const Point & cursor, Skill::Primary::skill_t & skill, const Rect & pos)
+{
+    if(Skill::Primary::UNKNOWN != skill)
+    {
+        Dialog::Message(Skill::Primary::String(skill), Skill::Primary::StringDescription(skill, hero), Font::BIG, Dialog::OK);
+	return true;
+    }
+
+    return false;
+}
+
+bool PrimarySkillsBar::ActionBarPressRight(const Point & cursor, Skill::Primary::skill_t & skill, const Rect & pos)
+{
+    if(Skill::Primary::UNKNOWN != skill)
+    {
+        Dialog::Message(Skill::Primary::String(skill), Skill::Primary::StringDescription(skill, hero), Font::BIG);
+	return true;
+    }
+
+    return false;
+}
+
+bool PrimarySkillsBar::ActionBarCursor(const Point & cursor, Skill::Primary::skill_t & skill, const Rect & pos)
+{
+    if(Skill::Primary::UNKNOWN != skill)
+    {
+	msg = _("View %{skill} Info");
+        StringReplace(msg, "%{skill}", Skill::Primary::String(skill));
+    }
+
+    return false;
+}
+
+bool PrimarySkillsBar::QueueEventProcessing(std::string* str)
+{
+    msg.clear();
+    bool res = Interface::ItemsBar<Skill::Primary::skill_t>::QueueEventProcessing();
+    if(str) *str = msg;
+    return res;
+}
+
+SecondarySkillsBar::SecondarySkillsBar(bool mini /* true */, bool change /* false */) : use_mini_sprite(mini), can_change(change)
+{
+    if(use_mini_sprite)
+    {
+	const Sprite & sprite = AGG::GetICN(ICN::HSICONS, 0);
+	const Rect rt(25, 20, 34, 34);
+	backsf.Set(rt.w, rt.h);
+	sprite.Blit(rt, 0, 0, backsf);
+	Cursor::DrawCursor(backsf, 0x70, true);
+	SetItemSize(rt.w, rt.h);
+    }
+    else
+    {
+	const Sprite & sprite = AGG::GetICN(ICN::SECSKILL, 0);
+	SetItemSize(sprite.w(), sprite.h());
+    }
+}
+
+void SecondarySkillsBar::RedrawBackground(const Rect & pos, Surface & dstsf)
+{
+    if(use_mini_sprite)
+	backsf.Blit(pos, dstsf);
+    else
+	AGG::GetICN(ICN::SECSKILL, 0).Blit(pos, dstsf);
+}
+
+void SecondarySkillsBar::RedrawItem(Skill::Secondary & skill, const Rect & pos, Surface & dstsf)
+{
+    if(skill.isValid())
+    {
+        const Sprite & sprite = use_mini_sprite ? 
+		AGG::GetICN(ICN::MINISS, skill.GetIndexSprite2()) : AGG::GetICN(ICN::SECSKILL, skill.GetIndexSprite1());
+        sprite.Blit(pos.x + (pos.w - sprite.w()) / 2, pos.y + (pos.h - sprite.h()) / 2, dstsf);
+
+	if(use_mini_sprite)
+	{
+    	    Text text(GetString(skill.Level()), Font::SMALL);
+    	    text.Blit(pos.x + (pos.w - text.w()) - 3, pos.y + pos.h - 12, dstsf);
+	}
+	else
+	{
+    	    Text text(Skill::Secondary::String(skill.Skill()), Font::SMALL);
+    	    text.Blit(pos.x + (pos.w - text.w()) / 2, pos.y + 3, dstsf);
+
+    	    text.Set(Skill::Level::String(skill.Level()));
+	    text.Blit(pos.x + (pos.w - text.w()) / 2, pos.y + 50, dstsf);
+	}
+    }
+}
+
+bool SecondarySkillsBar::ActionBarSingleClick(const Point & cursor, Skill::Secondary & skill, const Rect & pos)
+{
+    if(skill.isValid())
+    {
+	Dialog::SecondarySkillInfo(skill, true);
+	return true;
+    }
+    else
+    if(can_change)
+    {
+	Skill::Secondary alt = Dialog::SelectSecondarySkill();
+
+	if(alt.isValid())
+	{
+	    skill = alt;
+	    return true;
+        }
+    }
+
+    return false;
+}
+
+bool SecondarySkillsBar::ActionBarPressRight(const Point & cursor, Skill::Secondary & skill, const Rect & pos)
+{
+    if(skill.isValid())
+    {
+	if(can_change)
+	    skill.Reset();
+	else
+	    Dialog::SecondarySkillInfo(skill, false);
+	return true;
+    }
+
+    return false;
+}
+
+bool SecondarySkillsBar::ActionBarCursor(const Point & cursor, Skill::Secondary & skill, const Rect & pos)
+{
+    if(skill.isValid())
+    {
+	msg = _("View %{skill} Info");
+        StringReplace(msg, "%{skill}", skill.GetName());
+    }
+
+    return false;
+}
+
+bool SecondarySkillsBar::QueueEventProcessing(std::string* str)
+{
+    msg.clear();
+    bool res = Interface::ItemsBar<Skill::Secondary>::QueueEventProcessing();
+    if(str) *str = msg;
+    return res;
+}
